{
    "$graph": [
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "types": [
                        {
                            "type": "record",
                            "name": "#FASTQReadPairType.yml/FASTQReadPairType",
                            "fields": [
                                {
                                    "name": "#FASTQReadPairType.yml/FASTQReadPairType/name",
                                    "type": "string"
                                },
                                {
                                    "name": "#FASTQReadPairType.yml/FASTQReadPairType/read1_files",
                                    "type": {
                                        "type": "array",
                                        "items": "File"
                                    }
                                },
                                {
                                    "name": "#FASTQReadPairType.yml/FASTQReadPairType/read2_files",
                                    "type": {
                                        "type": "array",
                                        "items": "File"
                                    }
                                }
                            ]
                        }
                    ],
                    "class": "SchemaDefRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#main/bait_interval_list"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "id": "#main/interval_padding"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "id": "#main/intervals"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "secondaryFiles": [
                        ".idx"
                    ],
                    "id": "#main/known_sites"
                },
                {
                    "type": "string",
                    "id": "#main/library"
                },
                {
                    "type": "string",
                    "id": "#main/platform"
                },
                {
                    "type": "#FASTQReadPairType.yml/FASTQReadPairType",
                    "id": "#main/read_pair"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        ".fai",
                        "^.dict"
                    ],
                    "id": "#main/reference_genome"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".idx"
                    ],
                    "id": "#main/resource_dbsnp"
                },
                {
                    "type": "File",
                    "id": "#main/target_interval_list"
                },
                {
                    "type": "int",
                    "id": "#main/threads"
                }
            ],
            "steps": [
                {
                    "run": "#concat-gz-files.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "coresMin": 1,
                            "ramMin": 1024
                        }
                    ],
                    "scatter": [
                        "#main/combine_reads/files",
                        "#main/combine_reads/output_filename"
                    ],
                    "scatterMethod": "dotproduct",
                    "in": [
                        {
                            "source": "#main/file_pair_details/reads",
                            "id": "#main/combine_reads/files"
                        },
                        {
                            "source": "#main/generate_sample_filenames/combined_reads_output_filenames",
                            "id": "#main/combine_reads/output_filename"
                        }
                    ],
                    "out": [
                        "#main/combine_reads/output"
                    ],
                    "id": "#main/combine_reads"
                },
                {
                    "run": "#fastp-pe.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "coresMin": 4,
                            "ramMin": 2048
                        }
                    ],
                    "in": [
                        {
                            "source": "#main/combine_reads/output",
                            "id": "#main/fastp/reads"
                        },
                        {
                            "source": "#main/file_pair_details/read_pair_name",
                            "id": "#main/fastp/report_base_filename"
                        },
                        {
                            "default": 4,
                            "id": "#main/fastp/threads"
                        },
                        {
                            "source": "#main/generate_sample_filenames/trimmed_reads_output_filenames",
                            "id": "#main/fastp/trimmed_reads_filenames"
                        }
                    ],
                    "out": [
                        "#main/fastp/trimmed_reads",
                        "#main/fastp/html_report",
                        "#main/fastp/json_report"
                    ],
                    "id": "#main/fastp"
                },
                {
                    "run": "#extract-named-file-pair-details.cwl",
                    "in": [
                        {
                            "source": "#main/library",
                            "id": "#main/file_pair_details/library"
                        },
                        {
                            "source": "#main/platform",
                            "id": "#main/file_pair_details/platform"
                        },
                        {
                            "source": "#main/read_pair",
                            "id": "#main/file_pair_details/read_pair"
                        }
                    ],
                    "out": [
                        "#main/file_pair_details/reads",
                        "#main/file_pair_details/read_pair_name",
                        "#main/file_pair_details/read_group_header"
                    ],
                    "id": "#main/file_pair_details"
                },
                {
                    "run": "#generate-sample-filenames.cwl",
                    "in": [
                        {
                            "source": "#main/file_pair_details/read_pair_name",
                            "id": "#main/generate_sample_filenames/sample_name"
                        }
                    ],
                    "out": [
                        "#main/generate_sample_filenames/combined_reads_output_filenames",
                        "#main/generate_sample_filenames/trimmed_reads_output_filenames",
                        "#main/generate_sample_filenames/mapped_reads_output_filename",
                        "#main/generate_sample_filenames/sorted_reads_output_filename",
                        "#main/generate_sample_filenames/dedup_reads_output_filename",
                        "#main/generate_sample_filenames/dedup_metrics_output_filename",
                        "#main/generate_sample_filenames/recal_reads_output_filename",
                        "#main/generate_sample_filenames/recal_table_output_filename",
                        "#main/generate_sample_filenames/raw_variants_output_filename",
                        "#main/generate_sample_filenames/haplotypes_bam_output_filename"
                    ],
                    "id": "#main/generate_sample_filenames"
                },
                {
                    "run": "#gitc-bwa-mem-samtools.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "coresMin": "$(inputs.threads)",
                            "ramMin": 15336,
                            "outdirMin": 12288,
                            "tmpdirMin": 12288
                        }
                    ],
                    "in": [
                        {
                            "source": "#main/generate_sample_filenames/mapped_reads_output_filename",
                            "id": "#main/map/output_filename"
                        },
                        {
                            "source": "#main/file_pair_details/read_group_header",
                            "id": "#main/map/read_group_header"
                        },
                        {
                            "source": "#main/fastp/trimmed_reads",
                            "id": "#main/map/reads"
                        },
                        {
                            "source": "#main/reference_genome",
                            "id": "#main/map/reference"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/map/threads"
                        }
                    ],
                    "out": [
                        "#main/map/output_sorted_bam"
                    ],
                    "id": "#main/map"
                },
                {
                    "run": "#GATK4-MarkDuplicates.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "ramMin": 32768,
                            "outdirMin": 12288,
                            "tmpdirMin": 12288
                        }
                    ],
                    "in": [
                        {
                            "default": "coordinate",
                            "id": "#main/mark_duplicates/assume_sort_order"
                        },
                        {
                            "default": "true",
                            "id": "#main/mark_duplicates/create_index"
                        },
                        {
                            "source": "#main/map/output_sorted_bam",
                            "id": "#main/mark_duplicates/input_file"
                        },
                        {
                            "default": "-Xms4g",
                            "id": "#main/mark_duplicates/java_opt"
                        },
                        {
                            "source": "#main/generate_sample_filenames/dedup_metrics_output_filename",
                            "id": "#main/mark_duplicates/metrics_filename"
                        },
                        {
                            "default": 2500,
                            "id": "#main/mark_duplicates/optical_duplicate_pixel_distance"
                        },
                        {
                            "source": "#main/generate_sample_filenames/dedup_reads_output_filename",
                            "id": "#main/mark_duplicates/output_filename"
                        },
                        {
                            "default": "SILENT",
                            "id": "#main/mark_duplicates/validation_stringency"
                        }
                    ],
                    "out": [
                        "#main/mark_duplicates/output_dedup_bam_file",
                        "#main/mark_duplicates/output_metrics_file"
                    ],
                    "id": "#main/mark_duplicates"
                },
                {
                    "run": "#GATK4-BaseRecalibrator.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "ramMin": 12288
                        }
                    ],
                    "in": [
                        {
                            "source": "#main/mark_duplicates/output_dedup_bam_file",
                            "id": "#main/recalibrate_01_analyze/input_bam"
                        },
                        {
                            "source": "#main/interval_padding",
                            "id": "#main/recalibrate_01_analyze/interval_padding"
                        },
                        {
                            "source": "#main/intervals",
                            "id": "#main/recalibrate_01_analyze/intervals"
                        },
                        {
                            "default": "-Xms4g",
                            "id": "#main/recalibrate_01_analyze/java_opt"
                        },
                        {
                            "source": "#main/known_sites",
                            "id": "#main/recalibrate_01_analyze/known_sites"
                        },
                        {
                            "source": "#main/generate_sample_filenames/recal_table_output_filename",
                            "id": "#main/recalibrate_01_analyze/output_recalibration_report_filename"
                        },
                        {
                            "source": "#main/reference_genome",
                            "id": "#main/recalibrate_01_analyze/reference"
                        }
                    ],
                    "out": [
                        "#main/recalibrate_01_analyze/output_recalibration_report"
                    ],
                    "id": "#main/recalibrate_01_analyze"
                },
                {
                    "run": "#GATK4-ApplyBQSR.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "ramMin": 6144
                        }
                    ],
                    "in": [
                        {
                            "default": true,
                            "id": "#main/recalibrate_02_apply_bqsr/add_output_sam_program_record"
                        },
                        {
                            "source": "#main/recalibrate_01_analyze/output_recalibration_report",
                            "id": "#main/recalibrate_02_apply_bqsr/bqsr_report"
                        },
                        {
                            "source": "#main/mark_duplicates/output_dedup_bam_file",
                            "id": "#main/recalibrate_02_apply_bqsr/input_bam"
                        },
                        {
                            "source": "#main/interval_padding",
                            "id": "#main/recalibrate_02_apply_bqsr/interval_padding"
                        },
                        {
                            "source": "#main/intervals",
                            "id": "#main/recalibrate_02_apply_bqsr/intervals"
                        },
                        {
                            "default": "-Xms3g",
                            "id": "#main/recalibrate_02_apply_bqsr/java_opt"
                        },
                        {
                            "source": "#main/generate_sample_filenames/recal_reads_output_filename",
                            "id": "#main/recalibrate_02_apply_bqsr/output_recalibrated_bam_filename"
                        },
                        {
                            "source": "#main/reference_genome",
                            "id": "#main/recalibrate_02_apply_bqsr/reference"
                        }
                    ],
                    "out": [
                        "#main/recalibrate_02_apply_bqsr/output_recalibrated_bam"
                    ],
                    "id": "#main/recalibrate_02_apply_bqsr"
                },
                {
                    "run": "#GATK4-HaplotypeCaller.cwl",
                    "requirements": [
                        {
                            "class": "ResourceRequirement",
                            "coresMin": 1,
                            "ramMin": 18432
                        }
                    ],
                    "in": [
                        {
                            "default": [
                                "StandardAnnotation",
                                "AS_StandardAnnotation"
                            ],
                            "id": "#main/variant_calling/annotation_groups"
                        },
                        {
                            "default": "GVCF",
                            "id": "#main/variant_calling/emit_ref_confidence"
                        },
                        {
                            "source": "#main/recalibrate_02_apply_bqsr/output_recalibrated_bam",
                            "id": "#main/variant_calling/input_bam"
                        },
                        {
                            "source": "#main/interval_padding",
                            "id": "#main/variant_calling/interval_padding"
                        },
                        {
                            "source": "#main/intervals",
                            "id": "#main/variant_calling/intervals"
                        },
                        {
                            "default": "-Xms7g",
                            "id": "#main/variant_calling/java_opt"
                        },
                        {
                            "source": "#main/generate_sample_filenames/haplotypes_bam_output_filename",
                            "id": "#main/variant_calling/output_bam_filename"
                        },
                        {
                            "source": "#main/generate_sample_filenames/raw_variants_output_filename",
                            "id": "#main/variant_calling/output_variants_filename"
                        },
                        {
                            "source": "#main/reference_genome",
                            "id": "#main/variant_calling/reference"
                        }
                    ],
                    "out": [
                        "#main/variant_calling/output_variants",
                        "#main/variant_calling/output_bam"
                    ],
                    "id": "#main/variant_calling"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/fastp/html_report",
                    "id": "#main/fastp_html_report"
                },
                {
                    "type": "File",
                    "outputSource": "#main/fastp/json_report",
                    "id": "#main/fastp_json_report"
                },
                {
                    "type": "File",
                    "outputSource": "#main/variant_calling/output_bam",
                    "doc": "BAM file containing assembled haplotypes and locally realigned reads",
                    "id": "#main/haplotypes_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/mark_duplicates/output_dedup_bam_file",
                    "id": "#main/markduplicates_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/mark_duplicates/output_metrics_file",
                    "id": "#main/markduplicates_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/variant_calling/output_variants",
                    "doc": "VCF file from per sample variant calling",
                    "id": "#main/raw_variants"
                },
                {
                    "type": "File",
                    "outputSource": "#main/recalibrate_02_apply_bqsr/output_recalibrated_bam",
                    "id": "#main/recalibrated_reads"
                },
                {
                    "type": "File",
                    "outputSource": "#main/recalibrate_01_analyze/output_recalibration_report",
                    "id": "#main/recalibration_table"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.0.4.0"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "4.0.4.0"
                            ],
                            "https://schema.org/citation": "https://doi.org/10.1101/gr.107524.110",
                            "package": "gatk"
                        }
                    ]
                }
            ],
            "baseCommand": "gatk",
            "inputs": [
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "prefix": "--add-output-sam-program-record"
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/add_output_sam_program_record"
                },
                {
                    "type": "File",
                    "doc": "BQSR recalibration report",
                    "inputBinding": {
                        "prefix": "-bqsr"
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/bqsr_report"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-I"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "doc": "input bam",
                    "id": "#GATK4-ApplyBQSR.cwl/input_bam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Amount of padding (in bp) to add to each interval you are including. Defaults to 0",
                    "inputBinding": {
                        "prefix": "-ip"
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/interval_padding"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File",
                            "inputBinding": {
                                "prefix": "-L"
                            }
                        }
                    ],
                    "doc": "One or more genomic intervals over which to operate",
                    "id": "#GATK4-ApplyBQSR.cwl/intervals"
                },
                {
                    "type": "string",
                    "doc": "String of options to pass to JVM at runtime",
                    "inputBinding": {
                        "prefix": "--java-options",
                        "position": -1,
                        "shellQuote": true
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/java_opt"
                },
                {
                    "type": "string",
                    "doc": "Output BAM filename",
                    "default": "recalibrated.bam",
                    "inputBinding": {
                        "prefix": "-O"
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/output_recalibrated_bam_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-R"
                    },
                    "secondaryFiles": [
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        ".fai",
                        "^.dict"
                    ],
                    "id": "#GATK4-ApplyBQSR.cwl/reference"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "int",
                            "inputBinding": {
                                "prefix": "--static-quantized-quals"
                            }
                        }
                    ],
                    "doc": "Use static quantized quality scores to a given number of levels (with -bqsr)",
                    "id": "#GATK4-ApplyBQSR.cwl/static_quantized_quals"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "prefix": "--use-original-qualities"
                    },
                    "id": "#GATK4-ApplyBQSR.cwl/use_original_qualities"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_recalibrated_bam_filename)"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#GATK4-ApplyBQSR.cwl/output_recalibrated_bam"
                }
            ],
            "arguments": [
                {
                    "valueFrom": "ApplyBQSR",
                    "position": 0
                }
            ],
            "id": "#GATK4-ApplyBQSR.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.0.4.0"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "4.0.4.0"
                            ],
                            "https://schema.org/citation": "https://doi.org/10.1101/gr.107524.110",
                            "package": "gatk"
                        }
                    ]
                }
            ],
            "baseCommand": "gatk",
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-I"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "doc": "input bam",
                    "id": "#GATK4-BaseRecalibrator.cwl/input_bam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Amount of padding (in bp) to add to each interval you are including. Defaults to 0",
                    "inputBinding": {
                        "prefix": "-ip"
                    },
                    "id": "#GATK4-BaseRecalibrator.cwl/interval_padding"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File",
                            "inputBinding": {
                                "prefix": "-L"
                            }
                        }
                    ],
                    "doc": "One or more genomic intervals over which to operate",
                    "id": "#GATK4-BaseRecalibrator.cwl/intervals"
                },
                {
                    "type": "string",
                    "doc": "String of options to pass to JVM at runtime",
                    "inputBinding": {
                        "prefix": "--java-options",
                        "position": -1,
                        "shellQuote": true
                    },
                    "id": "#GATK4-BaseRecalibrator.cwl/java_opt"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File",
                            "inputBinding": {
                                "prefix": "--known-sites"
                            }
                        }
                    ],
                    "secondaryFiles": [
                        ".idx"
                    ],
                    "doc": "Any number of VCF files representing known SNPs and/or indels. Could be e.g. dbSNP and/or official 1000 Genomes indel calls. SNPs in these files will be ignored unless the --mismatchFraction argument is used. optional parameter.",
                    "id": "#GATK4-BaseRecalibrator.cwl/known_sites"
                },
                {
                    "type": "string",
                    "doc": "Recalibration Report Filename",
                    "default": "recal.csv",
                    "inputBinding": {
                        "prefix": "-O"
                    },
                    "id": "#GATK4-BaseRecalibrator.cwl/output_recalibration_report_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-R"
                    },
                    "secondaryFiles": [
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        ".fai",
                        "^.dict"
                    ],
                    "id": "#GATK4-BaseRecalibrator.cwl/reference"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "prefix": "--use-original-qualities"
                    },
                    "id": "#GATK4-BaseRecalibrator.cwl/use_original_qualities"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_recalibration_report_filename)"
                    },
                    "id": "#GATK4-BaseRecalibrator.cwl/output_recalibration_report"
                }
            ],
            "arguments": [
                {
                    "valueFrom": "BaseRecalibrator",
                    "position": 0
                }
            ],
            "id": "#GATK4-BaseRecalibrator.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.0.4.0"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "4.0.4.0"
                            ],
                            "https://schema.org/citation": "https://doi.org/10.1101/gr.107524.110",
                            "package": "gatk"
                        }
                    ]
                }
            ],
            "baseCommand": "gatk",
            "inputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "string",
                            "inputBinding": {
                                "prefix": "-G"
                            }
                        }
                    ],
                    "doc": "One or more groups of annotations to apply to variant calls  This argument may be specified 0 or more times. Default value: [StandardAnnotation, StandardHCAnnotation].",
                    "id": "#GATK4-HaplotypeCaller.cwl/annotation_groups"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "-D"
                    },
                    "secondaryFiles": [
                        ".idx"
                    ],
                    "doc": "dbSNP file. Default value: null",
                    "id": "#GATK4-HaplotypeCaller.cwl/dbsnp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "-ERC"
                    },
                    "doc": "Mode for emitting reference confidence scores  Default value: NONE. Possible values: {NONE, BP_RESOLUTION, GVCF}",
                    "id": "#GATK4-HaplotypeCaller.cwl/emit_ref_confidence"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-I"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "doc": "BAM/SAM/CRAM file containing reads. Required.",
                    "id": "#GATK4-HaplotypeCaller.cwl/input_bam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Amount of padding (in bp) to add to each interval you are including. Defaults to 0",
                    "inputBinding": {
                        "prefix": "-ip"
                    },
                    "id": "#GATK4-HaplotypeCaller.cwl/interval_padding"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File",
                            "inputBinding": {
                                "prefix": "-L"
                            }
                        }
                    ],
                    "doc": "One or more genomic intervals over which to operate",
                    "id": "#GATK4-HaplotypeCaller.cwl/intervals"
                },
                {
                    "type": "string",
                    "doc": "String of options to pass to JVM at runtime",
                    "inputBinding": {
                        "prefix": "--java-options",
                        "position": -1,
                        "shellQuote": true
                    },
                    "id": "#GATK4-HaplotypeCaller.cwl/java_opt"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "-bamout"
                    },
                    "doc": "File to which assembled haplotypes should be written.",
                    "id": "#GATK4-HaplotypeCaller.cwl/output_bam_filename"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "-O"
                    },
                    "doc": "File to which variants should be written  Required. Use .g.vcf to make a GVCF file.",
                    "id": "#GATK4-HaplotypeCaller.cwl/output_variants_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-R"
                    },
                    "secondaryFiles": [
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        ".fai",
                        "^.dict"
                    ],
                    "doc": "Reference sequence file  Required.",
                    "id": "#GATK4-HaplotypeCaller.cwl/reference"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_bam_filename)"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#GATK4-HaplotypeCaller.cwl/output_bam"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_variants_filename)"
                    },
                    "secondaryFiles": [
                        ".idx"
                    ],
                    "id": "#GATK4-HaplotypeCaller.cwl/output_variants"
                }
            ],
            "arguments": [
                {
                    "valueFrom": "HaplotypeCaller",
                    "position": 0
                }
            ],
            "id": "#GATK4-HaplotypeCaller.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.0.4.0"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "4.0.4.0"
                            ],
                            "https://schema.org/citation": "https://doi.org/10.1101/gr.107524.110",
                            "package": "gatk"
                        }
                    ]
                }
            ],
            "baseCommand": "gatk",
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--ASSUME_SORT_ORDER"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/assume_sort_order"
                },
                {
                    "type": "string",
                    "default": "false",
                    "inputBinding": {
                        "prefix": "--CREATE_INDEX"
                    },
                    "doc": "Create an index",
                    "id": "#GATK4-MarkDuplicates.cwl/create_index"
                },
                {
                    "type": "File",
                    "doc": "One or more input SAM or BAM files to analyze. Must be coordinate sorted.",
                    "inputBinding": {
                        "prefix": "--INPUT"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "String of options to pass to JVM at runtime",
                    "inputBinding": {
                        "prefix": "--java-options",
                        "position": -1,
                        "shellQuote": true
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/java_opt"
                },
                {
                    "type": "string",
                    "doc": "Output metrics filename",
                    "default": "marked_dup_metrics.txt",
                    "inputBinding": {
                        "prefix": "--METRICS_FILE"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/metrics_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "prefix": "--OPTICAL_DUPLICATE_PIXEL_DISTANCE"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/optical_duplicate_pixel_distance"
                },
                {
                    "type": "string",
                    "doc": "Output filename",
                    "default": "marked_duplicates.bam",
                    "inputBinding": {
                        "prefix": "--OUTPUT"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "If true do not write duplicates to the output file instead of writing them with appropriate flags set.  (Default true).",
                    "inputBinding": {
                        "prefix": "--REMOVE_DUPLICATES:",
                        "shellQuote": false
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/remove_duplicates"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--VALIDATION_STRINGENCY"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/validation_stringency"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#GATK4-MarkDuplicates.cwl/output_dedup_bam_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.metrics_filename)"
                    },
                    "id": "#GATK4-MarkDuplicates.cwl/output_metrics_file"
                }
            ],
            "arguments": [
                {
                    "valueFrom": "MarkDuplicates",
                    "position": 0
                }
            ],
            "id": "#GATK4-MarkDuplicates.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "combinefiles.sh",
                            "entry": "set -o pipefail\nif [ \"$#\" -eq 1 ]; then\n  cat \"$@\"\nelse\n  zcat \"$@\" | gzip\nfi\n"
                        }
                    ]
                }
            ],
            "baseCommand": "bash",
            "arguments": [
                "combinefiles.sh"
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 2,
                        "shellQuote": false
                    },
                    "id": "#concat-gz-files.cwl/files"
                },
                {
                    "type": "string",
                    "id": "#concat-gz-files.cwl/output_filename"
                }
            ],
            "stdout": "$(inputs.output_filename)",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#concat-gz-files.cwl/output"
                }
            ],
            "id": "#concat-gz-files.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/fastp:0.20.0--hdbcaa40_0"
                },
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "0.20.0"
                            ],
                            "package": "fastp",
                            "https://schema.org/citation": "https://doi.org/10.1093/bioinformatics/bty560"
                        }
                    ]
                }
            ],
            "inputs": [
                {
                    "type": "boolean",
                    "doc": "By default, the adapter sequence auto-detection is enabled for SE data only turn on this option to enable it for PE data",
                    "default": true,
                    "inputBinding": {
                        "prefix": "--detect_adapter_for_pe"
                    },
                    "id": "#fastp-pe.cwl/detect_adapter_for_pe"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "doc": "read1 and read2 input files array",
                    "id": "#fastp-pe.cwl/reads"
                },
                {
                    "type": "string",
                    "doc": "Prefix to use for HTML and JSON report files",
                    "id": "#fastp-pe.cwl/report_base_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "worker thread number, default is 2 (int [=2])",
                    "inputBinding": {
                        "prefix": "-w"
                    },
                    "id": "#fastp-pe.cwl/threads"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "doc": "Array of output file names. The output will be gzip-compressed if its file name ends with .gz",
                    "id": "#fastp-pe.cwl/trimmed_reads_filenames"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.report_base_filename + \".html\")"
                    },
                    "id": "#fastp-pe.cwl/html_report"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.report_base_filename + \".json\")"
                    },
                    "id": "#fastp-pe.cwl/json_report"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "$(inputs.trimmed_reads_filenames)"
                    },
                    "id": "#fastp-pe.cwl/trimmed_reads"
                }
            ],
            "baseCommand": "fastp",
            "arguments": [
                "-I",
                "$(inputs.reads[0])",
                "-i",
                "$(inputs.reads[1])",
                "-O",
                "$(inputs.trimmed_reads_filenames[0])",
                "-o",
                "$(inputs.trimmed_reads_filenames[1])",
                "-j",
                "$(inputs.report_base_filename + \".json\")",
                "-h",
                "$(inputs.report_base_filename + \".html\")"
            ],
            "id": "#fastp-pe.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/genomes-in-the-cloud:2.3.1-1512499786"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "bwa-mem-samtools.sh",
                            "entry": "set -xo pipefail\n/usr/gitc/bwa mem $@ | samtools sort -O BAM -\n"
                        }
                    ]
                }
            ],
            "baseCommand": "bash",
            "arguments": [
                "bwa-mem-samtools.sh"
            ],
            "hints": [
                {
                    "class": "SoftwareRequirement",
                    "packages": [
                        {
                            "version": [
                                "0.7.15"
                            ],
                            "package": "bwa",
                            "https://schema.org/citation": "https://dx.doi.org/10.1093/bioinformatics/btp324"
                        },
                        {
                            "version": [
                                "1.3.1"
                            ],
                            "package": "samtools",
                            "https://schema.org/citation": "https://doi.org/10.1093/bioinformatics/btp352"
                        }
                    ]
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "int"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-I",
                        "itemSeparator": ","
                    },
                    "id": "#gitc-bwa-mem-samtools.cwl/min_std_max_min"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-k"
                    },
                    "doc": "-k INT        minimum seed length [19]",
                    "id": "#gitc-bwa-mem-samtools.cwl/minimum_seed_length"
                },
                {
                    "type": "string",
                    "id": "#gitc-bwa-mem-samtools.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-R"
                    },
                    "doc": "-R STR        read group header line such as '@RG\\tID:foo\\tSM:bar' [null]",
                    "id": "#gitc-bwa-mem-samtools.cwl/read_group_header"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 3
                    },
                    "id": "#gitc-bwa-mem-samtools.cwl/reads"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "secondaryFiles": [
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa"
                    ],
                    "id": "#gitc-bwa-mem-samtools.cwl/reference"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "prefix": "-p"
                    },
                    "doc": "smart pairing (ignoring in2.fq)",
                    "id": "#gitc-bwa-mem-samtools.cwl/smart_pairing"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "prefix": "-Y"
                    },
                    "id": "#gitc-bwa-mem-samtools.cwl/soft_clipping"
                },
                {
                    "type": "int",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "-t INT        number of threads [1]",
                    "id": "#gitc-bwa-mem-samtools.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "prefix": "-v"
                    },
                    "doc": "verbose level: 1=error, 2=warning, 3=message, 4+=debugging [3]",
                    "id": "#gitc-bwa-mem-samtools.cwl/verbosity"
                }
            ],
            "stdout": "$(inputs.output_filename)",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename)"
                    },
                    "id": "#gitc-bwa-mem-samtools.cwl/output_sorted_bam"
                }
            ],
            "id": "#gitc-bwa-mem-samtools.cwl"
        },
        {
            "class": "ExpressionTool",
            "label": "Given a FASTQReadPairType returns a 2D array of the files contained within",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "types": [
                        {
                            "$import": "#FASTQReadPairType.yml/FASTQReadPairType"
                        }
                    ],
                    "class": "SchemaDefRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#extract-named-file-pair-details.cwl/library"
                },
                {
                    "type": "string",
                    "id": "#extract-named-file-pair-details.cwl/platform"
                },
                {
                    "type": "#FASTQReadPairType.yml/FASTQReadPairType",
                    "id": "#extract-named-file-pair-details.cwl/read_pair"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "id": "#extract-named-file-pair-details.cwl/read_group_header"
                },
                {
                    "type": "string",
                    "id": "#extract-named-file-pair-details.cwl/read_pair_name"
                },
                {
                    "type": {
                        "type": "array",
                        "items": {
                            "type": "array",
                            "items": "File"
                        }
                    },
                    "id": "#extract-named-file-pair-details.cwl/reads"
                }
            ],
            "expression": "${\n  var readPairName = inputs.read_pair.name;\n  var readGroupHeader = \"@RG\" +\n    \"\\\\tID:\" + readPairName +\n    \"\\\\tLB:\" + inputs.library +\n    \"\\\\tPL:\" + inputs.platform +\n    \"\\\\tPU:\" + readPairName +\n    \"\\\\tSM:\" + readPairName;\n  return {\n    reads: [\n      inputs.read_pair.read1_files,\n      inputs.read_pair.read2_files,\n    ],\n    read_pair_name: readPairName,\n    read_group_header: readGroupHeader\n  };\n\n}\n",
            "id": "#extract-named-file-pair-details.cwl"
        },
        {
            "class": "ExpressionTool",
            "label": "Generates a set of file names for preprocessing steps based on an input sample name",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/sample_name"
                }
            ],
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#generate-sample-filenames.cwl/combined_reads_output_filenames"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/dedup_metrics_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/dedup_reads_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/fixedtag_reads_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/haplotypes_bam_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/hs_metrics_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/mapped_reads_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/raw_variants_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/recal_reads_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/recal_table_output_filename"
                },
                {
                    "type": "string",
                    "id": "#generate-sample-filenames.cwl/sorted_reads_output_filename"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#generate-sample-filenames.cwl/trimmed_reads_output_filenames"
                }
            ],
            "expression": "${\n  function makeFilename(base, suffix, extension) {\n    return base + '-' + suffix + '.' + extension;\n  }\n  var base = inputs.sample_name\n\n  return {\n    combined_reads_output_filenames: [\n      makeFilename(base, 'R1', 'fastq.gz'),\n      makeFilename(base, 'R2', 'fastq.gz'),\n    ],\n    trimmed_reads_output_filenames: [\n      makeFilename(base, 'R1-trimmed', 'fastq.gz'),\n      makeFilename(base, 'R2-trimmed', 'fastq.gz'),\n    ],\n    mapped_reads_output_filename: makeFilename(base, 'mapped', 'bam'),\n    sorted_reads_output_filename: makeFilename(base, 'sorted', 'bam'),\n    dedup_reads_output_filename: makeFilename(base, 'dedup', 'bam'),\n    dedup_metrics_output_filename: makeFilename(base, 'dedup-metrics', 'out'),\n    recal_reads_output_filename: makeFilename(base, 'recal', 'bam'),\n    recal_table_output_filename: makeFilename(base, 'recal', 'table'),\n    raw_variants_output_filename: makeFilename(base, 'raw_variants', 'g.vcf'),\n    haplotypes_bam_output_filename:  makeFilename(base, 'haplotypes', 'bam'),\n    hs_metrics_output_filename: makeFilename(base, 'hs', 'txt'),\n    fixedtag_reads_output_filename: makeFilename(base, 'fixed_tags', 'bam')\n  };\n}\n",
            "id": "#generate-sample-filenames.cwl"
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "https://schema.org/version/latest/schema.rdf"
    ]
}
